@@ -2,8 +2,6 @@ __all__ = [
     "config_file",
     "dbmysql",
     "http_server",
-    "tencent_cos",
-    "tencent_duanxin",
     "token_mgr",
     "util",
     "wx"
