@@ -9,29 +9,43 @@
 import pymysql
 import logging
 import time
-
-from pymysql.constants.ER import NO
+import asyncio
+from typing import Dict, List, Union, Tuple
 
 
 class dbmysql:
-    def __init__(self, host: str, port: int, user: str, password: str, database: str):
-        self.conn_timestamp = time.time()
+    def __init__(self,
+                 host: str,
+                 port: int,
+                 user: str,
+                 password: str,
+                 database: str,
+                 loop: asyncio.AbstractEventLoop = None):
         self.conn = None
         self.host = host
         self.port = port
         self.user = user
         self.password = password
         self.database = database
-        self.tables = {}
-        self.timer = 0
-        self.privileges = {}
+        self.loop = loop or asyncio.get_event_loop()
+
+        # 权限，自动提交检测时间
+        self.periodic_elapse = 1
+
+        self.__tables = {}
+        self.__privileges = {}
+        self.__need_commit = False
+
         self.allow_operators = ["=",  "<>", ">",
                                 "<", ">=", "<=",
                                 "like", "in", "not in"]
         self.allow_values_func = ["from_unixtime", "unix_timestamp"]
-        self.functions = []
-        self.need_commit = False
+        
+        self.col_functions = []
+
         self.connect()
+        
+        self.__periodic_check()
 
     def __make_where(self, col_names: list, where: dict, inject: str = None):
         where_str = ""
@@ -75,7 +89,7 @@ class dbmysql:
                 columns.append(cols[i])
                 if need_index_list:
                     index_list.append(i)
-            elif cols[i] in self.functions:
+            elif cols[i] in self.col_functions:
                 ret_str += f"{cols[i]},"
                 columns.append(cols[i])
                 if need_index_list:
@@ -96,26 +110,20 @@ class dbmysql:
         """
         tables = self.get_table_names()
         for tn in tables:
-            self.tables[tn] = self.get_table_columns(tn)
+            self.__tables[tn] = self.get_table_columns(tn)
 
     def add_col_func(self, funcname):
-        self.functions.append(funcname)
+        self.col_functions.append(funcname)
 
-
-    def update_info(self, elapse: float):
+    def __periodic_check(self):
         """
-        帧更新函数
-            @elapse:float:上一帧执行耗时
+        @elapse 多长时间检测一次
         """
-        if self.need_commit:
+        if self.__need_commit:
             self.commit()
-            self.need_commit = False
-
-        self.timer += elapse
-        if self.timer >= 1:
-            self.update_privilege()
-            # self.flush_tables_cache()
-            self.timer = 0
+            self.__need_commit = False
+        self.update_privilege()
+        self.loop.call_later(self.periodic_elapse, self.__periodic_check)
 
     def get_table_names(self):
         """
@@ -160,35 +168,51 @@ class dbmysql:
             charset='utf8'
         )
         self.flush_tables_cache()
-    
+
     def commit(self):
         self.__execute_err_reconnect(self.__commit)
 
     def __commit(self):
         try:
             self.conn.commit()
-        except:
-            raise(Exception(1))
+        except Exception as e:
+            if e.args[0] == 2013:
+                raise(e)
+            return None
 
     def __execute_err_reconnect(self, fn, *args):
         try:
             ret = fn(*args)
         except Exception as e:
-            if str(e) == '1':
+            if e.args[0] == 2013:
                 try:
                     self.conn.ping()
-                except Exception:
-                    logging.error(f"ping mysql error")
+                    logging.info("Reconnecting to mysql ...")
+                except Exception as e:
+                    logging.error(e.with_traceback(None))
                     return None
                 ret = fn(*args)
             else:
                 return None
         return ret
 
-    def get(self, sql: str, params: any = {}) -> tuple:
+    def get(self, sql: str, params: Union[List, Tuple, Dict]) -> tuple:
+        """
+        从数据库获取数据
+            @sql:要执行的sql语句，%(key)s 或 %s 表示占位符
+            @params:dict:表示占位符的值。
+            @return:tuple:返回的结果
+        例如:
+            1.
+            sql = 'select * from `users` where `id` = %(id)s
+            params = {'id': 123}
+            2.
+            sql = 'select * from `users` where `id` = %s
+            params = (123)
+        """
         return self.__execute_err_reconnect(self.__get, sql, params)
 
-    def __get(self, sql: str, params: any = {}) -> tuple:
+    def __get(self, sql: str, params: Union[List, Tuple, Dict]) -> tuple:
         """
         从数据库获取数据
             @sql:要执行的sql语句，%(key)s 或 %s 表示占位符
@@ -206,7 +230,6 @@ class dbmysql:
             cursor: pymysql.cursors.Cursor = self.conn.cursor(
                 cursor=pymysql.cursors.SSDictCursor)
             ret = cursor.execute(sql, params)
-            self.need_commit = True
             if ret <= 0:
                 ret = None
             else:
@@ -218,7 +241,9 @@ class dbmysql:
         except Exception as e:
             logging.error(f"db get error {e.with_traceback(None)}")
             logging.error(f"get sql error:{sql},params = {str(params)}")
-            raise(Exception(1))
+            if e.args[0] == 2013:
+                raise(e)
+            return None
 
     def select(self, table: str, cols: list, where: dict, order_by: str = "",  limit: int = None, offset: int = None, order_direction: str = "asc", inject: str = ""):
         """
@@ -233,7 +258,7 @@ class dbmysql:
             @offset:int:从offset条数据后面返回limit条数据
         """
         # 获取表格名称
-        col_names = self.tables.get(table)
+        col_names = self.__tables.get(table)
         if not col_names:
             return
 
@@ -261,11 +286,22 @@ class dbmysql:
 
         return self.get(sql, args)
 
-    def post(self, sql: str, params: any = {}) -> tuple:
+    def post(self, sql: str, params: Union[List, Tuple, Dict]) -> tuple:
+        """
+        修改数据库数据
+            @sql:字符串:要执行的sql语句，%(key)s 或 %s 表示占位符
+            @params:表示占位符的值。
+        例如：
+            1.
+            sql = 'update `users` set `name` = %(name)s where `id` = %(id)s
+            params = {'id': 123,'name':'hahaha'}
+            2.
+            sql = 'update `users` set `name` = %s where `id` = %s
+            params = [123, 'hahaha']
+        """
         return self.__execute_err_reconnect(self.__post, sql, params)
 
-
-    def __post(self, sql: str, params: any = {}) -> bool:
+    def __post(self, sql: str, params: Union[List, Tuple, Dict]) -> bool:
         """
         修改数据库数据
             @sql:字符串:要执行的sql语句，%(key)s 或 %s 表示占位符
@@ -282,17 +318,34 @@ class dbmysql:
             cursor: pymysql.cursors.Cursor = self.conn.cursor()
             cursor.execute(sql, params)
             cursor.close()
-            self.need_commit = True
+            self.__need_commit = True
         except Exception as e:
             logging.error(e.with_traceback(None))
             logging.error(f"post sql error:{sql}, params = {str(params)}")
-            raise(Exception(1))
+            if e.args[0] == 2013:
+                raise(e)
+            return None
         return True
-    
-    def exec_proc(self, name: str, args: tuple = (), ret_count: tuple = ()) -> tuple:
+
+    def exec_proc(self, name: str, args: Union[List, Tuple] = (), ret_count: Union[List, Tuple] = ()) -> tuple:
+        """
+        执行存储过程
+            @name:str: 存储过程名None
+            @args:tuple: 存储过程参数
+            @ret_count:tuple:需要返回结果的索引
+        例如:
+            有存储过程create_folder需要三个参数(
+                foldername：varchar:in,
+                ownerid:varchar:in,
+                folderid:int:out)
+            我们需要执行create_folder并返回folderid
+            folderid = exec_proc(
+                'create_folder', ['folder_name','owner_id', 0], [2])
+            结果为folderid = (60,)
+        """
         return self.__execute_err_reconnect(self.__exec_proc, name, args, ret_count)
 
-    def __exec_proc(self, name: str, args: tuple = (), ret_count: tuple = ()) -> tuple:
+    def __exec_proc(self, name: str, args: Union[List, Tuple] = (), ret_count: Union[List, Tuple] = ()) -> tuple:
         """
         执行存储过程
             @name:str: 存储过程名None
@@ -316,12 +369,14 @@ class dbmysql:
             cursor.execute(s_result_sql)
             result = cursor.fetchone()
             cursor.close()
-            self.need_commit = True
+            self.__need_commit = True
             return result
         except Exception as e:
             logging.error(e.with_traceback(None))
             logging.error(f"excute procedure error:{name},args = {str(args)}")
-            raise(Exception(1))
+            if e.args[0] == 2013:
+                raise(e)
+            return None
 
     def update(self, table: str, values: dict, where: dict, values_func: dict = None, inject: str = ""):
         """
@@ -331,7 +386,7 @@ class dbmysql:
             @id_value:str:主键值
             @kwargs:dict:目标值的键值对
         """
-        col_names = self.tables.get(table)
+        col_names = self.__tables.get(table)
         if col_names is None:
             return
         sql = f"update `{table}` set "
@@ -373,7 +428,7 @@ class dbmysql:
                 "xxx =": 123
             }帅选条件
         """
-        col_names = self.tables.get(table)
+        col_names = self.__tables.get(table)
         if not col_names:
             return
         sql = f"delete from `{table}` "
@@ -389,7 +444,7 @@ class dbmysql:
             @values:list:值列表
         """
         # 获取列
-        col_names = self.tables.get(table)
+        col_names = self.__tables.get(table)
         if not col_names:
             return
 
@@ -445,7 +500,7 @@ class dbmysql:
             }
             @espire_in:int:id生效时常
         """
-        self.privileges[id] = {
+        self.__privileges[id] = {
             "timestamp": time.time() + espire_in,
             "privilege": privilege
         }
@@ -456,12 +511,11 @@ class dbmysql:
         """
         remove_list = []
         cur_time = time.time()
-        for id, privilege_data in self.privileges.items():
+        for id, privilege_data in self.__privileges.items():
             if cur_time >= privilege_data["timestamp"]:
                 remove_list.append(id)
-
         for id in remove_list:
-            del self.privileges[id]
+            del self.__privileges[id]
 
     def execute(self, args: dict):
         """
@@ -487,7 +541,7 @@ class dbmysql:
         if id is None:
             return {"errMsg": "No id not supported!"}
 
-        privilege: dict = self.privileges.get(args['id'])
+        privilege: dict = self.__privileges.get(args['id'])
         if privilege is None:
             return {"errMsg": "Permmison deny!"}
         privilege = privilege['privilege']
